"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.api = void 0;
const core_1 = require("@angular-devkit/core");
const schematics_1 = require("@angular-devkit/schematics");
const sync_request_1 = require("sync-request");
const R = require("ramda");
const PATH = require("path");
function api(options) {
    const { source_path, yapi_apiId } = options;
    const list = getApiList(options);
    const path = PATH.basename(process.cwd() + source_path);
    const filePath = core_1.join(path);
    return (tree, context) => {
        const ruleList = [];
        const optionsList = [];
        list.map(item => {
            const option = transform(options, item, path);
            ruleList.push(schematics_1.mergeWith(generate(option, path, tree), schematics_1.MergeStrategy.Overwrite));
            optionsList.push(option);
        });
        return schematics_1.branchAndMerge(schematics_1.chain([
            ...ruleList,
            cleanPath(filePath),
            addToApi(optionsList, filePath),
            createServerUrlConfig(options, filePath),
            updateConfig(yapi_apiId)
        ]))(tree, context);
    };
}
exports.api = api;
const getApiList = (options) => {
    const { yapi_projectId, yapi_apiId, yapi_serverUrl, yapi_token, yapi_uid } = options;
    const cookie = `_yapi_token=${yapi_token};_yapi_uid=${yapi_uid}`;
    const fetchData = (url) => {
        const res = sync_request_1.default("GET", url, {
            headers: {
                'Cookie': cookie
            }
        });
        const result = JSON.parse(res.getBody('utf8'));
        return result;
    };
    const fetchApi = ({ id, projectId }) => {
        const url = `${yapi_serverUrl}/api/interface/get?id=${id}`;
        const result = fetchData(url);
        if (result.errcode > 0 || !result.data) {
            console.error(url, result.errmsg);
        }
        else {
            apiResult.push({
                api: result.data,
                projectId
            });
        }
    };
    let apiResult = [];
    if (yapi_projectId) {
        const getApis = (id) => {
            const url = `${yapi_serverUrl}/api/plugin/export?type=json&pid=${id.id}&status=all&isWiki=false`;
            const result = fetchData(url);
            const list = R.flatten(R.pluck('list', result));
            if (list.length) {
                list.map(data => {
                    apiResult.push({
                        api: data,
                        projectId: id.projectId
                    });
                });
            }
            else {
                console.log(result);
            }
        };
        const projectIds = JSON.parse(decodeURI(yapi_projectId));
        const keys = projectIds ? R.keys(projectIds) : [];
        const projectIdsList = keys.length ? R.pipe(R.keys, R.map(key => R.map((value) => ({ id: value, projectId: key }))(projectIds[key])), R.flatten)(projectIds) : [];
        if (projectIdsList.length) {
            projectIdsList.map((item) => {
                getApis(item);
            });
        }
    }
    if (yapi_apiId) {
        const apiIds = JSON.parse(decodeURI(yapi_apiId));
        const keys = apiIds ? R.keys(apiIds) : [];
        const apiIdsList = keys.length ? R.pipe(R.keys, R.map(key => R.map((value) => ({ id: value, projectId: key }))(apiIds[key])), R.flatten)(apiIds) : [];
        if (apiIdsList.length) {
            apiIdsList.map((item) => {
                fetchApi(item);
            });
        }
    }
    return apiResult;
};
const transform = (source, data, path) => {
    const target = R.clone(source);
    const { api, projectId } = data;
    let inputs = [];
    if (api.req_query && api.req_query.length) {
        inputs = reduceQuery(api.req_query);
    }
    else if (api.req_body_other) {
        inputs = reduceRequestBodyOther(api.req_body_other);
    }
    else if (api.req_body_form && api.req_body_form.length) {
        inputs = reduceRequestBodyForm(api.req_body_form);
    }
    else if (api.req_params && api.req_params.length) {
        inputs = reduceRequestParams(api.req_params);
    }
    target.name = transformToName(api.path);
    target.title = api.title;
    target.path = transformToPath(api.path);
    target.id = api._id;
    target.method = api.method;
    target.projectId = projectId;
    target.project_id = api.project_id;
    target.inputs = inputs;
    target.responses = api.res_body ? reduceResponse(api.res_body) : [];
    target.isRestful = api.path.indexOf('{') >= 0 && api.path.indexOf('}') >= 0;
    target.filePath = core_1.join(path);
    return target;
};
const generate = (options, path, tree) => {
    return schematics_1.apply(schematics_1.url('./files'), [
        schematics_1.template(Object.assign(Object.assign({}, core_1.strings), options)),
        schematics_1.forEach((fileEntry) => {
            if (tree.exists(path + fileEntry.path)) {
                tree.overwrite(path + fileEntry.path, fileEntry.content);
            }
            ;
            return fileEntry;
        }),
        schematics_1.move(path),
    ]);
};
const typeMap = {
    'integer': 'number',
    'Integer': 'number',
    'int': 'number',
    'BigDecimal': 'number',
    'Long': 'number',
    'number': 'number',
    'array': 'any[]',
    'Array': 'any[]',
    'String': 'string',
    'string': 'string',
    'boolean': 'boolean',
    'Boolean': 'boolean'
};
const transformToName = (str) => {
    return str ? R.pipe(R.split('/'), R.map((item) => {
        item = item ? item.split('{').join('') : item;
        item = item ? item.split('}').join('') : item;
        return R.toUpper(item.charAt(0)) + R.toLower(item.substring(1));
    }), R.join(''))(str) : str;
};
const transformToPath = (str) => {
    return str ? R.pipe(R.split('/'), R.filter((item) => item.indexOf('{') < 0 && item.indexOf('}') < 0), R.join('/'))(str) : str;
};
const reduceRequestParams = (list) => {
    return R.map((item) => `
    ${item.name}: string;
  `)(list);
};
const reduceQuery = (list) => {
    const res = list && list.length ? R.pipe(R.flatten, R.map((item) => `
      /** ${item.desc} */
      ${item.name}${item.required === '1' ? '' : '?'}: ${typeMap[item.example] ? typeMap[item.example] : 'string'};
    `))(list) : [];
    return res;
};
const reduceRequestBodyOther = (str) => {
    if (str.indexOf('{') < 0) {
        return [];
    }
    const res = JSON.parse(str);
    const { properties, required } = res;
    const reduceItem = (item) => {
        return `
    /** ${properties[item].description} */
    ${item}${required && required[item] ? '' : '?'}: ${reduceType(properties[item].type)};`;
    };
    return properties ? R.pipe(R.keys, R.map(item => reduceItem(item)))(properties) : [];
};
const reduceRequestBodyForm = (forms = []) => {
    if (!forms.length) {
        return forms;
    }
    const reduceItem = (item) => {
        return `
  /** ${item.desc} */
  ${item.name}${item.required && item.required === '1' ? '' : '?'}: ${reduceType(item.type)};`;
    };
    return forms && forms.length ? R.map(item => reduceItem(item))(forms) : [];
};
const reduceType = (type) => {
    if (!type) {
        return 'unknown';
    }
    else {
        return typeMap[type] || 'unknown';
    }
};
const reduceResponse = (str) => {
    var _a;
    if (str.indexOf('{') < 0) {
        return '';
    }
    const res = JSON.parse(str);
    const { properties, required = {} } = ((_a = res.properties) === null || _a === void 0 ? void 0 : _a.data) || {};
    const reduceItem = (item) => {
        return `
    /** ${properties[item].description} */
    ${item}${required && required[item] ? '' : '?'}: ${reduceType(properties[item].type)};`;
    };
    return properties ? R.pipe(R.keys, R.map(item => reduceItem(item)))(properties) : [];
};
const addToApi = (optionsList, filePath) => {
    return (tree) => {
        let content = '';
        optionsList.map(option => {
            content += `
      // ${option.id} ${option.title}
  export * from './${option.method}${option.name}';`;
        });
        const apiPath = filePath + '/generated/api.ts';
        tree.create(apiPath, content);
        return tree;
    };
};
const cleanPath = (filePath) => {
    return (tree) => {
        try {
            if (tree.exists(`${filePath}/.DS_Store`)) {
                tree.delete(`${filePath}/.DS_Store`);
            }
            if (tree.exists(`${filePath}/generated/.DS_Store`)) {
                tree.delete(`${filePath}/generated/.DS_Store`);
            }
            if (tree.exists(`${filePath}/generated/api.ts`)) {
                tree.delete(`${filePath}/generated/api.ts`);
            }
            if (tree.exists(`${filePath}/generated/config.ts`)) {
                tree.delete(`${filePath}/generated/config.ts`);
            }
        }
        catch (err) {
            console.log(err);
        }
        return tree;
    };
};
const createServerUrlConfig = (options, filePath) => {
    const path = `${filePath}/generated/config.ts`;
    const ywConfigPath = PATH.join(process.cwd(), 'yw.config.json');
    const ywConfig = require(ywConfigPath);
    return (tree) => {
        const content = `
            export const configs: Record<string, any> = ${JSON.stringify(ywConfig.serverUrl_config)}
        `;
        tree.create(path, content);
        return tree;
    };
};
const updateConfig = (ids) => {
    const configPath = `yw.config.json`;
    return (tree) => {
        if (tree.exists(configPath)) {
            const data = tree.read(configPath);
            const result = JSON.parse(data.toString());
            result.yapi_apiId = JSON.parse(decodeURI(ids));
            tree.overwrite(configPath, JSON.stringify(result));
        }
        return tree;
    };
};
