import { join, dirname, extname, basename } from 'path';

import nodeResolver from '@rollup/plugin-node-resolve';
import replace from '@rollup/plugin-replace';
import json from '@rollup/plugin-json';
import image from '@rollup/plugin-image';
import clear from 'rollup-plugin-clear';
import progress from 'rollup-plugin-progress';
import externals from 'rollup-plugin-node-externals';
import postcss from 'rollup-plugin-postcss';
import typescript from 'rollup-plugin-typescript2';
import { terser } from 'rollup-plugin-terser';
import babel from '@rollup/plugin-babel';
import filesize from 'rollup-plugin-filesize';
import visualizer from 'rollup-plugin-visualizer';
import less from 'rollup-plugin-less';
import copy from 'rollup-plugin-copy';
import pkg from './package.json';

const isProd = process.env.NODE_ENV === 'production';
const extensions = ['.js', '.jsx', '.ts', '.tsx'];

const formats = {
  commonjs: {
    format: 'cjs',
    file: pkg.main,
    sourcemap: true,
    exports: 'named',
  },
  esm: {
    format: 'esm',
    file: pkg.module,
    sourcemap: true,
    exports: 'named',
  },
  umd: {
    format: 'umd',
    name: pkg.name,
    sourcemap: true,
    exports: 'named',
    globals: {
      antd: 'antd',
      react: 'React',
      'react-dom': 'ReactDOM',
      'react-is': 'ReactIs',
      'prop-types': 'PropTypes',
      'react-router-dom': 'ReactRouterDOM',
      'styled-components': 'styled',
    },
  },
};

const config = {
  input: './index.ts',
  inlineDynamicImports: true,
  output: [formats.esm, formats.commonjs],
  plugins: [
    clear({
      targets: ['lib'],
    }),
    progress({
      clearLine: false,
    }),
    replace({
    }),
    externals({
      deps: true,
    }),
    nodeResolver(),
    typescript(),
    image(),
    json(),
    postcss({
      use: [
        [
          'less',
          {
            javascriptEnabled: true,
          },
        ],
      ],
      plugins: [],
    }),
    babel({
      plugins: [
        ['import', { libraryName: '@ywfe/ywd', "libraryDirectory": "es", style: true }],
        ['@babel/plugin-proposal-decorators', { legacy: true }],
        [
          "styless",
          {
              "import": "~@ywfe/ywd/lib/styles/antd-theme.less",
              "lessOptions": {
                  "javascriptEnabled": true
              }
          }
        ]
      ]
    }),
    less(),
    copy({
      targets: [{ src: 'src/styles/**/*', dest: 'lib/styles/' }],
    }),
  ],
  watch: {
    include: 'src/**',
    exclude: 'node_modules/**',
  },
};

if (isProd) {
  const file = formats.commonjs.file;
  const ext = extname(file);

  config.output.push({
    ...formats.umd,
    file: join(dirname(file), `${basename(file, ext)}.umd${ext}`),
    plugins: [
      filesize(),
      visualizer({
        title: `${pkg.name} - ${pkg.author.name}`,
        filename: 'lib/bundle-analyzer-report.html',
      }),
    ],
  });

  config.output.push({
    ...formats.umd,
    file: join(dirname(file), `${basename(file, ext)}.umd.min${ext}`),
    plugins: [terser()],
  });
}

export default config;
